@@ -1,14 +1,135 @@
+# -*- coding: utf-8 -*-
+#!/usr/env/bin python
+
+import importlib
+import argparse
+import logging
 import os
+import sys
+import types
 
 import numpy as np
 from mud.funs import mud_sol, map_sol
 
+__author__ = "Mathematical Michael"
+__copyright__ = "Mathematical Michael"
+__license__ = "mit"
+from mud_examples import __version__
+from mud import __version__ as __mud_version__
+
+_logger = logging.getLogger(__name__)
+
+
+def parse_args(args):
+    """Parse command line parameters
+
+    Args:
+      args ([str]): command line parameters as list of strings
+
+    Returns:
+      :obj:`argparse.Namespace`: command line parameters namespace
+    """
+    
+    desc = """
+        Examples
+        """
+
+    parser = argparse.ArgumentParser(description=desc)
+    parser.add_argument('-e', '--example',       default='ode', type=str)
+    parser.add_argument('-m', '--num-measure',   default=[20, 100],  type=int, nargs='+')
+    parser.add_argument('-r', '--ratio-measure', default=[1],  type=float, nargs='+')
+    parser.add_argument('--num-trials',    default=20,    type=int)
+    parser.add_argument('-t', '--sensor-tolerance',  default=[0.1], type=float, nargs='+')
+    parser.add_argument('-s', '--seed',          default=21)
+    parser.add_argument('-lw', '--linewidth',    default=5)
+    parser.add_argument('--fsize',               default=32, type=int)
+    parser.add_argument('--bayes', action='store_true')
+    parser.add_argument('--alt', action='store_true')
+    parser.add_argument('--save', action='store_true')
+
+    parser.add_argument(
+        "--version",
+        action="version",
+        version=f"mud_examples {__version__}, mud {__mud_version__}")
+#     parser.add_argument('-n', '--num_samples',
+#         dest="num",
+#         help="Number of samples",
+#         default=100,
+#         type=int,
+#         metavar="INT")
+    parser.add_argument('-i', '--input_dim',
+        dest="input_dim",
+        help="Dimension of input space (default=2).",
+        default=2,
+        type=int,
+        metavar="INT")
+    parser.add_argument('-d', '--distribution',
+        dest="dist",
+        help="Distribution. `n` (normal), `u` (uniform, default)",
+        default='u',
+        type=str,
+        metavar="STR")
+#     parser.add_argument('-b', '--beta-params',
+#         dest="beta_params",
+#         help="Parameters for beta distribution. Overrides --distribution. (default = 1 1 )",
+#         default=None,
+#         nargs='+',
+#         type=float,
+#         metavar="FLOAT FLOAT")
+    parser.add_argument('-p', '--prefix',
+        dest="prefix",
+        help="Output filename prefix (no extension)",
+        default='results',
+        type=str,
+        metavar="STR")
+    parser.add_argument(
+        "-v",
+        "--verbose",
+        dest="loglevel",
+        help="set loglevel to INFO",
+        action="store_const",
+        const=logging.INFO)
+    parser.add_argument(
+        "-vv",
+        "--very-verbose",
+        dest="loglevel",
+        help="set loglevel to DEBUG",
+        action="store_const",
+        const=logging.DEBUG)
+    return parser.parse_args(args)
+
 
 def check_dir(directory):
     if not os.path.exists(directory):
         os.makedirs(directory)
 
-        
+
+class LazyLoader(types.ModuleType):
+
+  def __init__(self, module_name='utensor_cgen', submod_name=None):
+    self._module_name = '{}{}'.format(
+      module_name,
+      submod_name and '.{}'.format(submod_name) or ''
+    )
+    self._mod = None
+    super(LazyLoader, self).__init__(self._module_name)
+
+  def _load(self):
+    if self._mod is None:
+      self._mod = importlib.import_module(
+        self._module_name
+      )
+    return self._mod
+
+  def __getattr__(self, attrb):
+    try:
+        return getattr(self._load(), attrb)
+    except ModuleNotFoundError:
+        pass
+
+  def __dir__(self):
+    return dir(self._load())
+
 def fit_log_linear_regression(input_values, output_values):
     x, y = np.log10(input_values), np.log10(output_values)
     X, Y = np.vander(x, 2), np.array(y).reshape(-1, 1)
@@ -24,6 +145,7 @@ def experiment_measurements(fun, num_measurements, sd, num_trials, seed=21):
     experiments = {}
     solutions = {}
     for ns in num_measurements:
+        _logger.debug(f'Measurement experiment. Num measurements: {ns}')
         discretizations = []
         estimates = []
         for t in range(num_trials):
@@ -45,6 +167,7 @@ def experiment_equipment(fun, num_measure, sd_vals, num_trials, reference_value)
     sd_err = []
     sd_var = []
     for sd in sd_vals:
+        _logger.debug(f'Equipment Experiment. Std Dev: {sd}')
         temp_err = []
         for t in range(num_trials):
             _d = fun(sd=sd, num_obs=num_measure)
@@ -61,6 +184,7 @@ def extract_statistics(solutions, reference_value):
     means = []
     variances = []
     for ns in num_sensors_plot_conv:
+        _logger.debug(f'Extracting stats for {ns} measurements.')
         mud_solutions = solutions[ns]
         num_trials = len(mud_solutions)
         err = [np.linalg.norm(m - reference_value) for m in mud_solutions]
@@ -73,38 +197,20 @@ def extract_statistics(solutions, reference_value):
     return means, variances
 
 
-def experiment_measurements_index(fun, num_measurements, sd, num_trials, seed=21):
-    """
-    Fixed sensors, varying how much data is incorporated into the solution.
-    """
-    experiments = {}
-    solutions = {}
-    for ns in num_measurements:
-        ratios = []
-        mud_solutions = []
-        for t in range(num_trials):
-            np.random.seed(seed+t)
-            _r = fun(sd=sd, num_obs=ns)
-            ratios.append(_r)
-            mud_solutions.append(np.argmax(_r))
-        experiments[ns] = ratios
-        solutions[ns] = mud_solutions
-    
-    return experiments, solutions
+def make_2d_normal_mesh(N=50, window=1):
+    X = np.linspace(-window, window, N)
+    Y = np.linspace(-window, window, N)
+    X, Y = np.meshgrid(X, Y)
+    XX = np.vstack([X.ravel(), Y.ravel()]).T
+    return (X, Y, XX)
 
 
-def extract_statistics_index(x, solutions, reference_value):
-    num_sensors_plot_conv = solutions.keys()
-    means = []
-    variances = []
-    for ns in num_sensors_plot_conv:
-        mud_solutions = [ x[_index] for _index in solutions[ns] ]
-        err = np.linalg.norm(np.array(mud_solutions) - reference_value, axis=1)/np.sqrt(len(reference_value))
-        mean_mud_sol = np.mean(err)
-        var_mud_sol = np.var(err)
-        means.append(mean_mud_sol)
-        variances.append(var_mud_sol)
-    return means, variances
+def make_2d_unit_mesh(N=50, window=1):
+    X = np.linspace(0, window, N)
+    Y = np.linspace(0, window, N)
+    X, Y = np.meshgrid(X, Y)
+    XX = np.vstack([X.ravel(), Y.ravel()]).T
+    return (X, Y, XX)
 
 
 def compare_mud_map_pin(A, b, d, mean, cov):
